!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

MODULE mpiMOD
		
#if USEPARALELL==1

	use rulesMOD, only: &
	P_stdOut,           & 
	P_dateLength,       &
	P_dirLength,        &
	P_wordLength      
	USE utilsMOD, only: &
	tArg
	
	IMPLICIT NONE
	
	PRIVATE
	
	INCLUDE 'mpif.h'
	
	!LOCAL VARIABLES
	
	LOGICAL :: unitOpen
	INTEGER :: printUnit
		   
	PUBLIC :: create_mpi
	PUBLIC :: destroy_mpi
	PUBLIC :: messageC_mpi
	PUBLIC :: bCastT_mpi
	PUBLIC :: time_mpi
	
	CONTAINS

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE create_mpi(thisProc, nProc)
	
		INTEGER, INTENT(OUT) :: thisProc, nProc
	
		LOGICAL                               :: unitBusy
		CHARACTER(LEN=MPI_MAX_PROCESSOR_NAME) :: procName
		CHARACTER(LEN=25)                     :: nProcChar,     &
							 aProcChar,     &
							 c0,            &
							 c1,            &
							 printUnitName
		INTEGER                               :: ierr,          &
							 procNameLength
							 
			CALL MPI_INIT(ierr)
			
			IF(ierr .NE. MPI_SUCCESS)THEN
				WRITE(c0, '(I10)')ierr
				CALL error_mpi(' mpi_init ierr='//TRIM(ADJUSTL(c0)))
			END IF		
			
			CALL MPI_COMM_SIZE(MPI_COMM_WORLD, nProc, ierr)
			
			IF(ierr .NE. MPI_SUCCESS)THEN
				WRITE(c0, '(I10)')ierr			 
				CALL error_mpi(" mpi_size ierr="//TRIM(adjustl(c0)))
			END IF
		
			WRITE(nProcChar, '(I4.4)')nProc
		
			CALL MPI_COMM_RANK(MPI_COMM_WORLD, thisProc, ierr)
		
			IF(ierr .NE. MPI_SUCCESS)THEN
       				WRITE(c0,"(i10)") ierr
       				CALL error_mpi(" mpi_rank ierr="//TRIM(adjustl(c0)))
    			END IF
		
			WRITE(aProcChar, '(I4.4)')thisProc
		
			CALL MPI_GET_PROCESSOR_NAME(procName, procNameLength, ierr)
			
			IF(ierr .NE. MPI_SUCCESS)THEN
       				WRITE(c0,"(i10)") ierr
       				CALL error_mpi(" mpi_get_processor ierr="//TRIM(adjustl(c0)))
    			END IF

			DO printUnit = 0, 99
				INQUIRE(printUnit, OPENED=unitBusy)
				IF(.NOT. unitBusy) EXIT
			END DO
		
			printUnitName='Out.'//TRIM(aProcChar)//'.'//TRIM(nProcChar)
     				
			OPEN(UNIT=printUnit, FILE=TRIM(printUnitName), STATUS='REPLACE')
    			
			unitOpen =.TRUE.
				
			WRITE(c0,'(i10)') thisProc
			WRITE(c1,'(i10)') nProc
    			
			CALL messageC_mpi('process number '//TRIM(adjustl(c0))//&
          				  ' total '//TRIM(adjustl(c1))//&
         				  '; in '//TRIM(procName)//'.')
			
	END SUBROUTINE create_mpi

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE time_mpi(t)
	
		DOUBLE PRECISION, INTENT(OUT) :: t
		
			t = MPI_WTIME()
		
	END SUBROUTINE time_mpi
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
	SUBROUTINE destroy_mpi()
	
		INTEGER :: ierr
		
			CALL messageC_mpi('DESTROYING MPI')
			
			IF(unitOpen)THEN
				CLOSE(printUnit)
				unitOpen = .FALSE.
			END IF
			
			CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)
			
			CALL MPI_FINALIZE(ierr)
	
	END SUBROUTINE destroy_mpi

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE error_mpi(msg)
	
		CHARACTER(LEN=*), INTENT(IN) :: msg
		
			CALL messageC_mpi(msg)
			
			IF(unitOpen)THEN
				CLOSE(printUnit)
				unitOpen = .FALSE.
			END IF
			
			CALL MPI_ABORT(MPI_COMM_WORLD, -1)
			
			STOP
			
	END SUBROUTINE error_mpi
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE messageC_mpi(msg)
	
		CHARACTER(LEN=*), INTENT(IN) :: msg
		
			IF(unitOpen)THEN
				WRITE(printUnit, '(A)')msg
				CALL FLUSH(printUnit)
			END IF
			
			WRITE(P_stdOut, '(A)')msg
	
	END SUBROUTINE messageC_mpi
	
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	SUBROUTINE bCastT_mpi(iArg, root)
	
		INTEGER, INTENT(IN)       :: root
		TYPE(tArg), INTENT(INOUT) :: iArg		
		
		INTEGER(MPI_ADDRESS_KIND) :: address(16),       &
			   		     displacements(16)
		
		INTEGER :: typeId,            &
			   typelist(16),      &
			   block_len(16),     &			  
			   ierr	
	
			typelist(1)  = MPI_LOGICAL   ! chemMerge
			typelist(2)  = MPI_LOGICAL   ! ppbmConvert
		        typelist(3)  = MPI_CHARACTER ! dateChar
			typelist(4)  = MPI_CHARACTER ! atmosInputDir
			typelist(5)  = MPI_CHARACTER ! chemInputDir
			typelist(6)  = MPI_CHARACTER ! outDir
			typelist(7)  = MPI_CHARACTER ! atmosPrefix
			typelist(8)  = MPI_CHARACTER ! atmosSufix
			typelist(9)  = MPI_CHARACTER ! outPrefix
			typelist(10) = MPI_CHARACTER ! outSufix
			typelist(11) = MPI_INTEGER   ! atmosType                
			typelist(12) = MPI_INTEGER   ! outType
			typelist(13) = MPI_INTEGER   ! chemProj
			typelist(14) = MPI_INTEGER   ! steps
			typelist(15) = MPI_INTEGER   ! date
			typelist(16) = MPI_INTEGER   ! chemType
				 
			block_len(1)  = 1
			block_len(2)  = 1
			block_len(3)  = P_dateLength
			block_len(4)  = P_dirLength
			block_len(5)  = P_dirLength
			block_len(6)  = P_dirLength
			block_len(7)  = P_wordLength
			block_len(8)  = P_wordLength
			block_len(9)  = P_wordLength
			block_len(10) = P_wordLength			
			block_len(11) = 1
			block_len(12) = 1
			block_len(13) = 1
			block_len(14) = 2
			block_len(15) = 4 
			block_len(16) = 1
	
			CALL MPI_GET_ADDRESS(iArg%chemMerge	, address(1), ierr)
			CALL MPI_GET_ADDRESS(iArg%ppbmConvert	, address(2), ierr)
			CALL MPI_GET_ADDRESS(iArg%dateChar	, address(3), ierr)
			CALL MPI_GET_ADDRESS(iArg%atmosInputDir	, address(4), ierr)
			CALL MPI_GET_ADDRESS(iArg%chemInputDir	, address(5), ierr)
			CALL MPI_GET_ADDRESS(iArg%outDir 	, address(6), ierr)
			CALL MPI_GET_ADDRESS(iArg%atmosPrefix	, address(7), ierr)
			CALL MPI_GET_ADDRESS(iArg%atmosSufix	, address(8), ierr)
			CALL MPI_GET_ADDRESS(iArg%outPrefix	, address(9), ierr)
			CALL MPI_GET_ADDRESS(iArg%outSufix	, address(10), ierr)			
			CALL MPI_GET_ADDRESS(iArg%atmosType	, address(11), ierr)
			CALL MPI_GET_ADDRESS(iArg%outType       , address(12), ierr)
			CALL MPI_GET_ADDRESS(iArg%chemProj	, address(13), ierr)
			CALL MPI_GET_ADDRESS(iArg%steps		, address(14), ierr)
			CALL MPI_GET_ADDRESS(iArg%date		, address(15), ierr)
			CALL MPI_GET_ADDRESS(iArg%chemType	, address(16), ierr)
		
			displacements(1)  = address(1) - address(1)
			displacements(2)  = address(2) - address(1)
			displacements(3)  = address(3) - address(1)
			displacements(4)  = address(4) - address(1)
			displacements(5)  = address(5) - address(1)
			displacements(6)  = address(6) - address(1)
			displacements(7)  = address(7) - address(1)
			displacements(8)  = address(8) - address(1)
			displacements(9)  = address(9) - address(1)
			displacements(10) = address(10) - address(1)
			displacements(11) = address(11) - address(1)
			displacements(12) = address(12) - address(1)
			displacements(13) = address(13) - address(1)
			displacements(14) = address(14) - address(1)
			displacements(15) = address(15) - address(1)
			displacements(16) = address(16) - address(1)
	
			CALL MPI_TYPE_CREATE_STRUCT(16, block_len, displacements, typelist, typeId, ierr)
			CALL MPI_TYPE_COMMIT(typeId, ierr)

			CALL MPI_BCAST(iArg, 1, typeId, root, MPI_COMM_WORLD, ierr)
			
			CALL MPI_TYPE_FREE(typeId, ierr)
	
	END SUBROUTINE bCastT_mpi

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

#endif

END MODULE mpiMOD
